// PortAudio.dll und PortAudioSharp.dll neben die .cs-Dateien legen.

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using PortAudioSharp;

namespace PortAudioDemo
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private static long t = 0;
        private const int delay = 20000;
        private static float[,] buffer = new float[delay, 2];

        private static PortAudio.PaStreamCallbackResult work(IntPtr input, IntPtr output, uint framesPerBuffer, ref PortAudio.PaStreamCallbackTimeInfo timeInfo, PortAudio.PaStreamCallbackFlags flags, IntPtr userData)
        {
            unsafe
            {
                float* inbuffer = (float*)input;
                float* outbuffer = (float*)output;

                for (int s = 0; s < framesPerBuffer; s++)
                {
                    for (int c = 0; c < 2; c++)
                    {
                        float outSample = buffer[(t + 1) % delay, c];
                        outbuffer[2 * s + c] = 10.0f * outSample;
                        float inSample = inbuffer[2 * s + c] + 0.5f * outSample; // Echo
                        buffer[t, c] = inSample; // mit Feedback
                    }
                    t++;
                    if (t >= delay)
                    {
                        t = 0;
                    }
                }
            }

            return PortAudio.PaStreamCallbackResult.paContinue;
        }

        private static void CheckError(PortAudio.PaErrorCode error)
        {
            if (error != PortAudio.PaErrorCode.paNoError)
            {
                throw new Exception(PortAudio.Pa_GetErrorText(error));
            }
        }
        private static IntPtr stream;
        private static PortAudio.PaStreamCallbackDelegate audioCallback = new PortAudio.PaStreamCallbackDelegate(work);

        private bool isRunning = false;

        private void startStopButton_Click(object sender, EventArgs e)
        {
            if (!isRunning)
            {
                CheckError(PortAudio.Pa_Initialize());

                int numDevices = PortAudio.Pa_GetDeviceCount();
                System.Diagnostics.Debug.WriteLine("Found " + numDevices + " devices:");
                for (int i = 0; i < numDevices; i++)
                {
                    PortAudio.PaDeviceInfo info = PortAudio.Pa_GetDeviceInfo(i);
                    System.Diagnostics.Debug.WriteLine("#" + i + ": " + info.name + ", " + Enum.GetName(typeof(PortAudio.PaHostApiTypeId), info.hostApi + 1) + ", " + info.defaultLowOutputLatency + ", " + info.maxInputChannels + ", " + info.maxOutputChannels);
                }

                PortAudio.PaStreamParameters inputParameters = new PortAudio.PaStreamParameters();
                inputParameters.channelCount = 2;
                inputParameters.device = PortAudio.Pa_GetDefaultInputDevice();
                inputParameters.sampleFormat = PortAudio.PaSampleFormat.paFloat32;

                PortAudio.PaStreamParameters outputParameters = new PortAudio.PaStreamParameters();
                outputParameters.channelCount = 2;
                outputParameters.device = PortAudio.Pa_GetDefaultOutputDevice();
                outputParameters.sampleFormat = PortAudio.PaSampleFormat.paFloat32;

                CheckError(PortAudio.Pa_IsFormatSupported(ref inputParameters, ref outputParameters, 48000.0));
                CheckError(PortAudio.Pa_OpenStream(out stream, ref inputParameters, ref outputParameters, 48000.0,
                    2048U, PortAudio.PaStreamFlags.paNoFlag, audioCallback, IntPtr.Zero));
                CheckError(PortAudio.Pa_StartStream(stream));
                isRunning = true;
            }
            else
            {
                CheckError(PortAudio.Pa_StopStream(stream));
                stream = IntPtr.Zero;
                CheckError(PortAudio.Pa_Terminate());
                isRunning = false;
            }
        }
    }
}